# frozen_string_literal: true

require_relative '../../migrations'
require_relative 'console'

module Ros
  module Core
    class Engine < ::Rails::Engine
      config.generators.api_only = true
      config.generators do |g|
        g.test_framework :rspec, fixture: true
        g.fixture_replacement :factory_bot, dir: 'spec/factories'
      end

      # Adds this gem's db/migrations path to the enclosing application's migraations_path array
      # https://github.com/rails/rails/issues/22261
			initializer :append_migrations do |app|
				config.paths['db/migrate'].expanded.each do |expanded_path|
					app.config.paths['db/migrate'] << expanded_path
					ActiveRecord::Migrator.migrations_paths << expanded_path
				end
			end

			initializer :jsonapi_authorization do |app|
        JSONAPI.configure do |config|
          config.default_processor_klass = JSONAPI::Authorization::AuthorizingProcessor
          config.exception_class_whitelist = [Pundit::NotAuthorizedError]
        end
      end

      # NOTE: ENV vars indicate hierarchy with two underscores '__'
      # export PLATFORM__CREDENTIALS__JWT_ENCRYPTION_KEY='test'
			initializer :platform_settings do |app|
        settings_path = root.join('config/settings')
        # NOTE: Sources are prepended in reverse order, meaning the first prepend is loaded last
        Settings.prepend_source!({ credentials: Rails.application.credentials.config })
        Settings.prepend_source!("#{settings_path}.yml")
        Settings.reload!
      end

			initializer :platform_services_connections do |app|
        connection_type = Settings.dig(:services, :connection, :type)
        client_config = Settings.dig(:services, :connection, connection_type).to_h
        Ros::Platform::Client.configure(client_config.merge(connection_type: connection_type))
      end

			initializer :load_middleware do |app|
        Warden::Strategies.add(:api_token, Ros::ApiTokenStrategy)
        app.config.middleware.use Warden::Manager do |manager|
          manager.default_strategies :api_token
          manager.failure_app = Ros::FailureApp
        end
        app.config.middleware.use Ros::TenantMiddleware
      end

      initializer :documentation do |app|
				require 'open_api'
				OpenApi::Config.class_eval do
					# Part 1: configs of this gem
					self.file_output_path = 'public/open_api'

					# Part 2: config (DSL) for generating OpenApi info
					open_api :doc_name, base_doc_classes: [ApplicationDoc]
					info version: '1.0.0', title: 'Platform APIs'#, description: ..
					# server 'http://localhost:3000', desc: 'Internal staging server for testing'
					bearer_auth :Authorization
				end
      end

      # Configure any error reporting services if their credential has been set
      # For now, only sentry.io is supported
      initializer :error_reporting_services do |app|
        # export PLATFORM__CREDENTIALS__SENTRY_DSN=url
        if Settings.credentials.sentry_dsn
          require 'sentry-raven'
          Raven.configure do |config|
            config.dsn = Settings.credentials.sentry_dsn
          end
        end
      end

      # Add console methods from ./console.rb
      config.after_initialize do
        TOPLEVEL_BINDING.eval('self').extend Ros::Console::Methods
      end
    end
  end
end
